#   NetLabsLibs -- a Python Framework for Business Objects
#
#   Copyright (C) 2003  net-labs Systemhaus GmbH
#                       info@net-labs.de
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
#   $Header: /data/cvs/libs/nll/src/nll/database/sqldm/interfaces.py,v 1.5 2003/05/02 18:33:32 jack Exp $
#   $Revision: 1.5 $

from peak.api import *
from protocols import Attribute

__all__ = ['ISQLEntityDM', 'ISQLQueryDM']

class ISQLEntityDM(storage.IWritableDM):

    # Attributes
    # TODO: Make most of these private.
    connection = Attribute('A SQL Connection')
    table = Attribute('A table name')
    fields = Attribute('A tuple of tuples: ((DB field name, model feature it corresponds to), ...)')
    primaryKey = Attribute('A tuple of field names which comprise the primary key')
    readOnlyFields = Attribute("A tuple of the names of fields we shouldn't write to the DB")
    autoIncrementedField = Attribute("The name of one field that is set by a DB-side autoincrement mechanism, or None")
    
    sequenceSpec = Attribute('A tuple of tuples: ((DB field name, name of sequence to fill it from), ...)')

    # Methods
    def oidForRow(row):
        """Return the oid that `row`'s corresponding Element would have.
        
        `row` -- a DB row with (at least) all the fields this DM needs
        """
    
    def stateForRow(row):
        """Take raw row from the DB, and return a state dict that can be used to populate the model Element."""

    def dbValuesFromElement(ob):
        """Given a model Element, return the list of DB-native values that represent its saved state. Values are formatted such that they are bindable via the PEAK SQL API, and the list is parallel to self.__class__writableFields. (Omit nonwritable fields: for example, autoincrement fields and DB-generated timestamps.)"""


class ISQLQueryDM(storage.IKeyableDM):

    querySpec = Attribute('A SQL Query Specification')
